@@ -3,10 +3,20 @@ use strict;
 use warnings;
 use Math::Calc::Parser;
 use Encode ();
-our $VERSION = '1.002';
+our $VERSION = '1.003';
+my ($bignum, $bigrat);
+sub import {
+	my $class = shift;
+	foreach my $opt (@_) {
+		$bignum = 1 if $opt eq '-bignum';
+		$bigrat = 1 if $opt eq '-bigrat';
+	}
+}
+sub unimport { $bignum = $bigrat = undef }
 use Filter::Simple sub {
 	my $expr = quotemeta Encode::decode 'UTF-8', "$_", Encode::FB_CROAK | Encode::LEAVE_SRC;
-	$_ = 'use utf8; print Math::Calc::Parser::calc "' . $expr . '", "\n";'
+	my $opts = $bigrat ? 'bigrat => 1' : $bignum ? 'bignum => 1' : '';
+	$_ = "use utf8; print Math::Calc::Parser->new($opts)->evaluate(qq{$expr}), qq{\n};"
 };
 1;
 
@@ -23,11 +33,14 @@ ath - Evaluate mathematical expressions in a compact one-liner
   $ perl -Math -e'round e^(i*pi)'
   $ perl -Math -e'log 5rand'
   $ perl -Math -e'2π'
+  $ perl -Math=-bignum -e'30!'
+  $ perl -Math=-bigrat -e'3/9'
 
 =head1 DESCRIPTION
 
 A source filter that parses and evaluates the source code as a mathematical
-expression using L<Math::Calc::Parser>, and prints the result.
+expression using L<Math::Calc::Parser>, and prints the result. The C<-bignum>
+or C<-bigrat> flags may be passed to activate the corresponding option.
 
 =head1 BUGS
 
